import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:medapay/entity.dart';
import 'package:medapay/src/components/feedback_message.dart';
import 'package:medapay/src/components/payment_account.dart';
import 'package:medapay/src/components/payment_action_bar.dart';
import 'package:medapay/src/components/payment_options.dart';
import 'package:medapay/src/components/preloader.dart';
import 'package:medapay/src/components/reference_number.dart';
import 'package:medapay/src/components/section_title.dart';
import 'package:medapay/src/constants/errors.dart';
import 'package:medapay/src/entity/medapay_payment.dart';
import 'package:medapay/src/exceptions/MedaPayException.dart';
import 'package:medapay/src/screen/medapay_amole.dart';
import 'package:medapay/src/screen/medapay_boa.dart';
import 'package:medapay/src/service/medapay_service.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:webview_flutter/webview_flutter.dart';

class MedaPayMainScreen extends StatefulWidget {
  final String paymentTitle;
  final MedaPayPayment payment;
  final String referenceNumber;
  final MedaPayService service;
  final Function onReferenceNumber;
  final Function onError;
  final Function onSuccess;
  final Function onCancel;
  final Function onBillInfo;
  final List<String> paymentMethods;
  final String preferedPaymentMethod;

  const MedaPayMainScreen(
      {Key key,
      @required this.paymentTitle,
      @required this.payment,
      @required this.referenceNumber,
      @required this.service,
      this.onReferenceNumber,
      this.onError,
      this.onSuccess,
      this.onCancel,
      this.onBillInfo,
      this.paymentMethods,
      this.preferedPaymentMethod})
      : super(key: key);

  @override
  _MedaPayMainScreenState createState() => _MedaPayMainScreenState();
}

class _MedaPayMainScreenState extends State<MedaPayMainScreen> {
  String referenceNumber;
  double amount;
  String accountNumber;
  String customerName;
  String description;
  String paymentMethod;
  bool isBusy;
  bool hasError;
  String errorMessage;
  Map<String, dynamic> billData;
  MedaPayError error;

  @override
  void initState() {
    super.initState();
    paymentMethod = widget.preferedPaymentMethod;
    hasError = false;
    isBusy = true;
    billData = null;
    errorMessage = '';
    error = null;

    if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();

    Future.delayed(Duration.zero, () async {
      if (widget.referenceNumber == 'n/a') {
        await _createBill();
      } else {
        await _fetchBill();
      }
    });
  }

  _createBill() async {
    try {
      var response = await widget.service.createBill(this.widget.payment);
      setState(() {
        isBusy = false;
      });
      if (response.statusCode == HttpStatus.created) {
        var decodeBody = jsonDecode(response.body);
        setState(() {
          hasError = false;
          errorMessage = '';
          billData = decodeBody;
          referenceNumber = decodeBody['billReferenceNumber'];
          accountNumber = widget.payment.accountNumber;
          customerName = widget.payment.customerName;
          amount = widget.payment.amount;
          description = widget.payment.description;
        });
        widget.onReferenceNumber(decodeBody['billReferenceNumber']);
      } else
        throw MedaPayException(new MedaPayError(
            code: ErrorCodes.billCreateError,
            shortMessage: ErrorShortMessages.billCreateError,
            longMessage: ErrorLongMessages.billCreateError));
    } on Exception catch (e) {
      widget.onError(e is MedaPayException
          ? e.error
          : MedaPayError(
              code: 99999,
              shortMessage: 'Oops: Something went wrong',
              longMessage: 'An error occurred while fetching bill'));
      setState(() {
        isBusy = false;
        hasError = true;
        errorMessage = 'An error occurred while creating bill.';
        billData = null;
        error = e is MedaPayException
            ? e.error
            : MedaPayError(
                code: 99999,
                shortMessage: 'Oops: Something went wrong',
                longMessage: 'An error occurred while creating bill');
        ;
      });
    }
  }

  _fetchBill() async {
    try {
      var response =
          await widget.service.fetchBill(this.widget.referenceNumber);
      setState(() {
        isBusy = false;
      });
      if (response.statusCode == HttpStatus.ok) {
        var decodeBody = jsonDecode(response.body);
        if (decodeBody['status'] == 'PENDING') {
          setState(() {
            hasError = false;
            errorMessage = '';
            billData = decodeBody;
            referenceNumber = decodeBody['referenceNumber'];
            accountNumber = decodeBody['accountNumber'];
            customerName = decodeBody['customerName'];
            amount = double.parse('${decodeBody['amount']}');
            description = decodeBody['description'];
          });
        } else {
          widget.onBillInfo(MedaPayBillInfo.fromDecodedBody(decodeBody));
          throw MedaPayException(new MedaPayError(
              code: ErrorCodes.billStatusError,
              shortMessage: 'Bill has been ${decodeBody['status']}',
              longMessage:
                  'Bill has been ${decodeBody['status']} on ${DateTime.parse(decodeBody['updatedAt']).toString()}'));
        }
      } else
        throw MedaPayException(new MedaPayError(
            code: ErrorCodes.billFetchError,
            shortMessage: ErrorShortMessages.billFetchError,
            longMessage: ErrorLongMessages.billFetchError));
    } catch (e) {
      widget.onError(e is MedaPayException
          ? e.error
          : MedaPayError(
              code: 99999,
              shortMessage: 'Oops: Something went wrong',
              longMessage: 'An error occurred while fetching bill'));
      setState(() {
        isBusy = false;
        hasError = true;
        errorMessage = 'An error occurred while fetching bill.';
        billData = null;
        error = e is MedaPayException
            ? e.error
            : MedaPayError(
                code: 99999,
                shortMessage: 'Oops: Something went wrong',
                longMessage: 'An error occurred while fetching bill');
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var _backIcon = Platform.isIOS ? Icons.arrow_back_ios : Icons.arrow_back;

    return WillPopScope(
      onWillPop: () async => false,
      child: Scaffold(
        backgroundColor: Color(0xFFF1F5F9),
        appBar: AppBar(
          title: Image.asset(
              'packages/medapay/assets/images/medapay-appbar-logo.png',
              fit: BoxFit.cover),
          backgroundColor: Color(0xfff56a05),
          leading: Container(),
          centerTitle: true,
        ),
        body: billData != null
            ? Stack(
                children: [
                  Container(
                    color: Colors.white,
                    child: ListView(
                      children: [
                        PaymentAccount(
                          customerName: customerName,
                          accountNumber: accountNumber,
                        ),
                        SizedBox(
                          height: 20,
                        ),
                        SectionTitle(
                          title: "Reference Number",
                          subtitle: "This is a unique payment identifier",
                        ),
                        ReferenceNumber(
                          number: referenceNumber,
                        ),
                        SizedBox(
                          height: 10,
                        ),
                        SectionTitle(
                          title: "Payment Method",
                          subtitle: "Choose your payment method",
                        ),
                        PaymentOptions(
                          value: paymentMethod,
                          paymentMethods: widget.paymentMethods,
                          onSelected: (selectedValue) {
                            this.setState(() {
                              paymentMethod = selectedValue;
                            });
                          },
                        ),
                        SizedBox(
                          height: 220,
                        ),
                      ],
                    ),
                  ),
                  PaymentActionBar(
                    title: widget.paymentTitle,
                    amount: amount,
                    description: description,
                    onCancel: widget.onCancel,
                    onContinue: () {
                      if (paymentMethod == MedaPayPaymentMethods.AMOLE) {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => MedaPayAmoleScreen(
                                      service: widget.service,
                                      referenceNumber: referenceNumber,
                                      onBillInfo: widget.onBillInfo,
                                    )));
                      } else if (paymentMethod ==
                          MedaPayPaymentMethods.ABYSSINIA_BANK) {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => MedaPayBOAScreen(
                                      service: widget.service,
                                      referenceNumber: referenceNumber,
                                      onBillInfo: widget.onBillInfo,
                                    )));
                      } else if (paymentMethod ==
                          MedaPayPaymentMethods.CBE_BIRR) {
                        launch('tel://*847*5*2*717171*$referenceNumber\%23');
                      }
                    },
                    isActive: paymentMethod.length > 0,
                  ),
                ],
              )
            : error != null
                ? FeedbackMessage(
                    icon: Icons.error_outlined,
                    shortMessage: error.shortMessage,
                    longMessage: error.longMessage,
                    type: FeedbackMessageType.info,
                    action: () {
                      Navigator.pop(context);
                    },
                    actionTitle: 'Back to ' + widget.paymentTitle,
                    actionIcon: Icons.arrow_back,
                  )
                : Preloader(
                    title: 'Loading',
                    subtitle: 'Please wait while loading...',
                  ),
      ),
    );
  }
}
